/*************** <lalVerbatim file="ArrayFactoriesTestCV"> ****
$Id: ArrayFactoriesTest.m4,v 1.5 2000/09/08 23:19:24 jolien Exp $
**************** </lalVerbatim> *******************************/

/* <lalLaTeX>

\subsection{Program \texttt{ArrayFactoriesTest.c}}
\label{ss:ArrayFactoriesTest.c}

A program to test create/destroy array routines.

\subsubsection*{Usage}
\begin{verbatim}
ArrayFactoriesTest [options]
Options:
  -h         print help
  -q         quiet: run silently
  -v         verbose: print extra information
  -d level   set lalDebugLevel to level
\end{verbatim}

\subsubsection*{Description}

\subsubsection*{Exit codes}
\begin{tabular}{|c|l|}
\hline
 Code & Explanation                   \\
\hline
\tt 0 & Success, normal exit.         \\
\tt 1 & Subroutine failed.            \\
\hline
\end{tabular}

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
lalDebugLevel
TYPECODECreateArray()
TYPECODEDestroyArray()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{ArrayFactoriesTestCV}}

</lalLaTeX> */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "LALConfig.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include "LALStdlib.h"
#include "AVFactories.h"

#define CODES_(x) #x
#define CODES(x) CODES_(x)

NRCSID( MAIN, "$Id: ArrayFactoriesTest.m4,v 1.5 2000/09/08 23:19:24 jolien Exp $" );

extern char *optarg;
extern int   optind;

int lalDebugLevel = 1;
int verbose    = 0;

static void
Usage (const char *program, int exitflag);

static void
ParseOptions (int argc, char *argv[]);

static void
TestStatus (LALStatus *status, const char *expectedCodes, int exitCode);

static void
ClearStatus (LALStatus *status);



















void ZArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static COMPLEX16Array  *array;
  static COMPLEX16Array   astore;
  static COMPLEX16    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALZCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( COMPLEX16 ) );

  LALZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALZCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALZCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALZCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALZDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALZCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALZCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALZDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALZCreateArray and LALZDestroyArray \n" );
          
  return;
}




















void CArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static COMPLEX8Array  *array;
  static COMPLEX8Array   astore;
  static COMPLEX8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALCCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( COMPLEX8 ) );

  LALCDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALCCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALCCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALCCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALCDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALCCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALCDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALCCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALCDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALCDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALCCreateArray and LALCDestroyArray \n" );
          
  return;
}




















void DArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static REAL8Array  *array;
  static REAL8Array   astore;
  static REAL8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALDCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( REAL8 ) );

  LALDDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALDCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALDCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALDCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALDDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALDCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALDDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALDCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALDDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALDDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALDCreateArray and LALDDestroyArray \n" );
          
  return;
}




















void SArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static REAL4Array  *array;
  static REAL4Array   astore;
  static REAL4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALSCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( REAL4 ) );

  LALSDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALSCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALSCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALSCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALSDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALSCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALSDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALSCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALSDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALSDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALSCreateArray and LALSDestroyArray \n" );
          
  return;
}




















void I2ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static INT2Array  *array;
  static INT2Array   astore;
  static INT2    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALI2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( INT2 ) );

  LALI2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALI2CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI2CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALI2CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALI2DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI2CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALI2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALI2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALI2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALI2CreateArray and LALI2DestroyArray \n" );
          
  return;
}




















void I4ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static INT4Array  *array;
  static INT4Array   astore;
  static INT4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALI4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( INT4 ) );

  LALI4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALI4CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI4CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALI4CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALI4DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI4CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALI4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALI4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALI4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALI4CreateArray and LALI4DestroyArray \n" );
          
  return;
}




















void I8ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static INT8Array  *array;
  static INT8Array   astore;
  static INT8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALI8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( INT8 ) );

  LALI8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALI8CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI8CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALI8CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALI8DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI8CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALI8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALI8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALI8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALI8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALI8CreateArray and LALI8DestroyArray \n" );
          
  return;
}




















void U2ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static UINT2Array  *array;
  static UINT2Array   astore;
  static UINT2    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALU2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( UINT2 ) );

  LALU2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALU2CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU2CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALU2CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALU2DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU2CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALU2CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALU2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALU2DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALU2CreateArray and LALU2DestroyArray \n" );
          
  return;
}




















void U4ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static UINT4Array  *array;
  static UINT4Array   astore;
  static UINT4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALU4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( UINT4 ) );

  LALU4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALU4CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU4CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALU4CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALU4DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU4CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALU4CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALU4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALU4DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALU4CreateArray and LALU4DestroyArray \n" );
          
  return;
}




















void U8ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static UINT8Array  *array;
  static UINT8Array   astore;
  static UINT8    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALU8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( UINT8 ) );

  LALU8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALU8CreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU8CreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALU8CreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALU8DestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU8CreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALU8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALU8CreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALU8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALU8DestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALU8CreateArray and LALU8DestroyArray \n" );
          
  return;
}




















void ArrayFactoriesTest ( void )
{
  static UINT4   dims[3]    = { 1, 2, 4 };
  static UINT4   dbad[3]    = { 1, 0, 4 };
  UINT4Vector    dimLength  = { 3, dims };
  UINT4Vector    badLength1 = { 3, NULL };
  UINT4Vector    badLength2 = { 0, dims };
  UINT4Vector    badLength3 = { 3, dbad };
  static LALStatus  status;
  static REAL4Array  *array;
  static REAL4Array   astore;
  static REAL4    datum;


  /*
   *
   * Test ordinary behavior.
   *
   */


  LALCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( 0 ), 1 );

  memset( array->data, 0, dims[0]*dims[1]*dims[2]*sizeof( REAL4 ) );

  LALDestroyArray ( &status, &array );
  TestStatus( &status, CODES( 0 ), 1 );

  LALCheckMemoryLeaks();


  /*
   *
   * Test error codes.
   *
   */


  LALCreateArray ( &status, &array, &badLength1 );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALCreateArray ( &status, &array, &badLength2 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );

  LALCreateArray ( &status, &array, &badLength3 );
  TestStatus( &status, CODES( AVFACTORIESH_ELENGTH ), 1 );
  LALCheckMemoryLeaks();

  LALDestroyArray ( &status, NULL );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALCreateArray ( &status, NULL, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EVPTR ), 1 );

  LALDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  array = &astore;
  LALCreateArray ( &status, &array, &dimLength );
  TestStatus( &status, CODES( AVFACTORIESH_EUPTR ), 1 );

  LALDestroyArray ( &status, &array );
  TestStatus( &status, CODES( AVFACTORIESH_EDPTR ), 1 );

  array->data = &datum;
  LALDestroyArray ( &status, &array );
  TestStatus( &status, CODES( -1 ), 1 );
  ClearStatus( &status );

  LALCheckMemoryLeaks();
  printf( "PASS... tests of LALCreateArray and LALDestroyArray \n" );
          
  return;
}


int main( int argc, char *argv[] )
{
  ParseOptions( argc, argv );

  ArrayFactoriesTest();
  ZArrayFactoriesTest();
  CArrayFactoriesTest();
  DArrayFactoriesTest();
  SArrayFactoriesTest();
  I2ArrayFactoriesTest();
  I4ArrayFactoriesTest();
  I8ArrayFactoriesTest();
  U2ArrayFactoriesTest();
  U4ArrayFactoriesTest();
  U8ArrayFactoriesTest();

  return 0;
}


/*
 * TestStatus ()
 *
 * Routine to check that the status code status->statusCode agrees with one of
 * the codes specified in the space-delimited string ignored; if not,
 * exit to the system with code exitcode.
 *
 */
static void
TestStatus (LALStatus *status, const char *ignored, int exitcode)
{
  char  str[64];
  char *tok;

  if (verbose)
  {
    REPORTSTATUS (status);
  }

  if (strncpy (str, ignored, sizeof (str)))
  {
    if ((tok = strtok (str, " ")))
    {
      do
      {
        if (status->statusCode == atoi (tok))
        {
          return;
        }
      }
      while ((tok = strtok (NULL, " ")));
    }
    else
    {
      if (status->statusCode == atoi (tok))
      {
        return;
      }
    }
  }

  fprintf (stderr, "\nExiting to system with code %d\n", exitcode);
  exit (exitcode);
}


/*
 *
 * ClearStatus ()
 *
 * Recursively applies DETATCHSTATUSPTR() to status structure to destroy
 * linked list of statuses.
 *
 */
void
ClearStatus (LALStatus *status)
{
  if (status->statusPtr)
  {
    ClearStatus      (status->statusPtr);
    DETATCHSTATUSPTR (status);
  }
}


/*
 * Usage ()
 *
 * Prints a usage message for program program and exits with code exitcode.
 *
 */
static void
Usage (const char *program, int exitcode)
{
  fprintf (stderr, "Usage: %s [options]\n", program);
  fprintf (stderr, "Options:\n");
  fprintf (stderr, "  -h         print this message\n");
  fprintf (stderr, "  -q         quiet: run silently\n");
  fprintf (stderr, "  -v         verbose: print extra information\n");
  fprintf (stderr, "  -d level   set lalDebugLevel to level\n");
  exit (exitcode);
}


/*
 * ParseOptions ()
 *
 * Parses the argc - 1 option strings in argv[].
 *
 */
static void
ParseOptions (int argc, char *argv[])
{
  while (1)
  {
    int c = -1;

    c = getopt (argc, argv, "hqvd:");
    if (c == -1)
    {
      break;
    }

    switch (c)
    {
      case 'd': /* set debug level */
        lalDebugLevel = atoi (optarg);
        break;

      case 'v': /* verbose */
        ++verbose;
        break;

      case 'q': /* quiet: run silently (ignore error messages) */
        freopen ("/dev/null", "w", stderr);
        freopen ("/dev/null", "w", stdout);
        break;

      case 'h':
        Usage (argv[0], 0);
        break;

      default:
        Usage (argv[0], 1);
    }

  }

  if (optind < argc)
  {
    Usage (argv[0], 1);
  }

  return;
}

